#!/usr/bin/env python3
# encoding: utf-8

from bs4 import BeautifulSoup
from slugify import slugify
from markdownify import MarkdownConverter
import os
import re
import json

in_file = 'wpexport20241219.xml'


class IframeConverter(MarkdownConverter):
	def convert_iframe(self, el, text, convert_as_inline):
		src = el['src']
		if 'youtube' in src:
			code = re.search(r'embed/([^?]+)', src).group(1)
			return f"::: youtube {code}\n:::\n"

		if 'steam' in src:
			code = re.search(r'\/([0-9]+)', src).group(1)
			return f"::: steam {code}\n:::\n"

		return f"::: iframe {src}\n:::\n"


def mkdirp(path):
	if not os.path.exists(path):
		os.makedirs(path)


def md(html, **options):
	return IframeConverter(**options).convert(html)


def code_parser(el):
	if not el.has_attr('data-language'):
		return 'text'

	lang = el['data-language']
	if lang == 'jscript':
		return 'js'

	return lang


mkdirp('output')

with open(in_file, 'r', encoding='utf-8') as f:
	raw_content = f.read()

soup = BeautifulSoup(raw_content, 'xml')

items = soup.find_all('item')
print(f"{len(items)} items found")

for item in items:
	post_type_raw = item.find('wp:post_type', recursive=False)
	if post_type_raw is None:
		continue

	post_type = post_type_raw.contents[0]
	if post_type != 'page' and post_type != 'post':
		continue

	post_id = item.find('wp:post_id', recursive=False).contents[0]
	title = item.find('title', recursive=False).contents[0]
	creator = item.find('dc:creator', recursive=False).contents[0]
	date = item.find('wp:post_date', recursive=False).contents[0]
	status = item.find('wp:status', recursive=False).contents[0]
	tags = item.findAll('category', {'domain': 'post_tag'})
	tags = '\n'.join(list(map(lambda x: ' - ' + x.contents[0], tags)))
	categories = item.findAll('category', {'domain': 'category'})
	categories = '\n'.join(
		list(map(lambda x: ' - ' + x.contents[0], categories)))
	content = item.find('content:encoded', recursive=False).contents[0]
	content = re.sub(r'<!-- /?wp:((?!syntaxhighlighter).)*\s*-->', '', content)
	content = re.sub(r'<!--more-->', '<p class="more">:::more:::</p>', content)
	lines = [s for s in content.splitlines()]
	for i in range(len(lines)):
		line = lines[i]
		if line.startswith('<!-- wp:syntaxhighlighter/code'):
			possibleParams = line[30:-4].strip()
			if possibleParams == '':
				continue
			params = json.loads(possibleParams)
			nextline = lines[i + 1]
			if 'language' in params:
				nextline = nextline[
						   :4] + f" data-language={params['language']} " + nextline[
																		   4:]
			if 'highlightLines' in params:
				nextline = nextline[
						   :4] + f" data-highlight={params['highlightLines']} " + nextline[
																				  4:]
			lines[i + 1] = nextline

	content = '\n'.join(lines)
	content = md(content, code_language_callback=code_parser).strip() + '\n'
	frontmatter = f"---\nlayout: {post_type}\nid: {post_id}\ntitle: \"{title}\"\ncreator: \"{creator}\"\ndate: {date}\ncategories:\n{categories}\ntags:\n{tags}\ndraft: {'true' if status == 'draft' else 'false'}\npublished: {'true' if status == 'publish' else 'false'}\n---\n\n"

	title_slug = slugify(title)
	if len(title_slug) < 1:
		title_slug = 'untitled'

	output_folder = 'output/' + str(date)[:4] + '/' + str(date)[5:7] + '/'
	mkdirp(output_folder)
	filename_candidate = output_folder + title_slug + ".md"
	filename_disambiguation = 1

	while os.path.exists(filename_candidate):
		filename_candidate = "output/" + title_slug + "_(" + str(
			filename_disambiguation) + ").md"
		filename_disambiguation = filename_disambiguation + 1

	with open(filename_candidate, 'w', encoding='utf-8') as f:
		f.write(frontmatter)
		f.write(content)
